import debug from 'debug';

import { productionConfig } from './config.prod';

// get the local configuration, if there is one...
let localConfig = { default: {} };
try {
    const isRunningTests = process && !process.browser;
    if (!isRunningTests) {
        localConfig = require('./config.local');
    }
}
catch (err) { /* intentional */ }

const fullVersion = chrome.runtime.getManifest().version;
const versionParts = fullVersion.split('.');
const version = `${versionParts[0]}.${versionParts[1]}`;

// build our config by combining the production configuration and
// any local configuration that is available...
const config = {
    version,
    ...productionConfig,
    ...localConfig.default,
};

// manually merge features specifically (spread operator is shallow)
if (productionConfig.features && localConfig.default.features) {
    config.features = {
        ...productionConfig.features,
        ...localConfig.default.features,
    };
}

// environment specific settings
const environmentMap = {
    production: {
        ext_env: '',
    },
    development: {
        ext_env: 'dev-',
        sniplets: 'https://dev-sniplets.bombbomb.com',
        metrics: 'https://metro.bombbomb.io/metric',
        metroLogging: 'https://metro.bombbomb.io/log',
        featureUsage: 'https://metro.bombbomb.io/featureUsage',
        livefire: 'https://dev-livefire.bombbomb.com',
        app: 'https://dev.app.bombbomb.com',
        api: 'https://dev.api.bombbomb.com',
    },
    local: {
        app: 'https://local.app.bombbomb.com',
        api: 'https://local.api.bombbomb.com',
        ext_env: 'local.',
    },
};

// environment based feature flag values...
const environmentFeatures = {
    development: {
        subscriptionCheck: true,
        loginDotCom: true,
        menuTeamLink: true,
        menuSubscriptionLink: true,
    },
};

const currentEnvironment = environmentMap[config.env];
for (const key in currentEnvironment) {
    if (currentEnvironment.hasOwnProperty(key)) {
        const element = currentEnvironment[key];
        config[key] = element;
    }
}

for (const featureFlag in environmentFeatures[config.env]) {
    if (environmentFeatures[config.env].hasOwnProperty(featureFlag)) {
        config.features[featureFlag] = environmentFeatures[config.env][featureFlag];
    }
}

// enable the configured debug selector, https://www.npmjs.com/package/debug#conventions
if (process.env.NODE_ENV !== 'test' && config.debug) {
    debug.enable(config.debug);
}

export default config; // eslint-disable-line import/no-default-export

export const featureFlagMap = {
    // map web app feature flags to chrome feature flags here
    // eg. SomeWebAppFlag: 'myFlag'
    ChromeLivefireBadgeNotifications: 'ChromeLivefireBadgeNotifications',
};

/**
 * Check if a feature flag is enabled in config or local storage.
 */
export function hasFeatureFlag(flag) {
    if (config.features && typeof config.features[flag] !== 'undefined' && config.features[flag] !== false) {
        return config.features[flag];
    }
    if (config.env !== 'production' && localStorage) {
        const localFeatures = JSON.parse(localStorage.getItem('local-features') || '{}');
        if (localFeatures && localFeatures[flag]) {
            return localFeatures[flag];
        }
    }
    return false;
}
